/***************************************************************************
                          ext_wav_play_object_impl.cpp  -  description
                             -------------------
    begin                : Son Jul 21 2002
    copyright            : (C) 2002 by Thomas Friedrichsmeier
    email                : tfry@users.sourceforge.net
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "config.h"
#define HAVE_LIBAUDIOFILE

#include "arts/soundserver.h"
#include "arts/cachedwav.h"
#include "arts/stdsynthmodule.h"
#include "arts/convert.h"
#include "arts/debug.h"
#include <math.h>

#include "ext_wav_play_object.h"

using namespace std;
using namespace Arts;

class EXT_WavPlayObject_impl : virtual public EXT_WavPlayObject_skel, virtual public StdSynthModule {
protected:
	CachedWav *wav;
	double flpos;
	poState _state;

	bool _loop;
	float _speed;

	int sampleCount()
	{
		if(!wav) return 0;
		return wav->bufferSize / wav->channelCount / (wav->sampleWidth/8);
	}

	bool loop() { return _loop; }
	void loop(bool newValue) { _loop = newValue; }
	float speed() { return _speed; }
	void speed(float newValue) { _speed = newValue; }

public:
	/*
	 * construction, destruction
	 */
	EXT_WavPlayObject_impl() :wav(0), flpos(0.0), _state(posIdle)
	{
		arts_debug("EXT_WavPlayObject_impl");
	}

	virtual ~EXT_WavPlayObject_impl()
	{
		arts_debug("~EXT_WavPlayObject_impl");
		if(wav) wav->decRef();
	}

	/*
	 * KMedia2 private interface
	 */

	bool loadMedia(const string &filename) {
		arts_debug("Wav: loadMedia %s", filename.c_str());
		wav = CachedWav::load(Cache::the(), filename);
		speed (1);
		loop (false);
		return (wav != 0);
	}

	/*
	 * KMedia2 interface
	 */
	string description() {
		return "no description (see wavplayobject_impl.cc)";
	}

	poTime currentTime() {
		if(!wav) return poTime(0,0,0,"samples");

		float timesec = flpos / (float)wav->samplingRate / _speed;
		float timems = (timesec - floor(timesec)) * 1000.0;

		return poTime((long) timesec, (long) timems, (int)flpos, "samples");
	}

    poTime overallTime() {
		float timesec = (float)sampleCount() / (float)wav->samplingRate;
		float timems = (timesec - floor(timesec)) * 1000.0;

		return poTime((long) timesec, (long) timems, sampleCount(), "samples");
	}

	poCapabilities capabilities() {
  		return static_cast<poCapabilities>(capPause+capSeek);
	}

	string mediaName() {
		return "";
	}

	poState state() {
		return _state;
	}

	void play() {
		_state = posPlaying;
	}

	void halt() {
		_state = posIdle;
		flpos = 0.0;
	}

	void seek(const class poTime &newTime) {
		if(!wav) return;

		float fnewsamples = -1;
		if(newTime.seconds != -1 && newTime.ms != -1)
		{
			float flnewtime = (float)newTime.seconds+((float)newTime.ms/1000.0);
			fnewsamples = flnewtime * (float)wav->samplingRate * _speed;
		}
		else if(newTime.custom >= 0 && newTime.customUnit == "samples")
		{
			fnewsamples = newTime.custom * _speed;
		}

		if(fnewsamples > (float)sampleCount())
			fnewsamples = (float)sampleCount();

		if(fnewsamples >= 0) flpos = fnewsamples;
	}

	void pause() {
		arts_debug("pause");
		_state = posPaused;
	}

	void calculateBlock(unsigned long samples) {
		unsigned long haveSamples = 0;

		if(wav && _state == posPlaying)
		{
			double speed = wav->samplingRate / samplingRateFloat * _speed;

			haveSamples = uni_convert_stereo_2float(samples, wav->buffer,
		   		wav->bufferSize,wav->channelCount,wav->sampleWidth,
		   		left,right,speed,flpos);

			flpos += (double)haveSamples * speed;
		}

		if(haveSamples != samples)
		{
			if (_loop) {
				do {
					flpos = 0;
					double speed = wav->samplingRate / samplingRateFloat * _speed;
					haveSamples += uni_convert_stereo_2float(samples - haveSamples, wav->buffer,
				   		wav->bufferSize,wav->channelCount,wav->sampleWidth,
				   		left,right,speed,flpos);
				} while (haveSamples != samples);
			} else {
				unsigned long i;

				for(i=haveSamples;i<samples;i++)
					left[i] = right[i] = 0.0;

				if(_state == posPlaying) {
					_state = posIdle;
					flpos = 0.0;
				}
			}
		}
	}
};

REGISTER_IMPLEMENTATION(EXT_WavPlayObject_impl);
