/***************************************************************************
                          savegames.cpp  -  description
                             -------------------
    begin                : Wed Jun 17 2004
    copyright            : (C) 2004, 2005 by Thomas Friedrichsmeier
    email                : tfry@users.sourceforge.net
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/
#include "savegames.h"

#include <kconfig.h>
#include <kapplication.h>
#include <klocale.h>

#include "cdp.h"

#include "defines.h"

SaveGames::SaveGames () {
	saves_list.setAutoDelete (true);
	readList ();
}

SaveGames::~SaveGames () {
	saves_list.clear ();
}

void SaveGames::addSaveGame (const QString &mission_file, const QString &mission_title, const QString &mission_version, int level, int lives, float fuel, float score) {
	GameDescription *descr = new GameDescription;
	descr->mission_file = mission_file;
	descr->mission_title = mission_title;
	descr->mission_version = mission_version;
	descr->level = level;
	descr->lives = lives;
	descr->fuel = fuel;
	descr->score = score;
	
	saves_list.append (descr);
	saveList ();
}

void SaveGames::readList () {
	saves_list.clear ();

	KConfig *config = kapp->config();

	config->setGroup ("SavedGames");
	
	unsigned int num;
	QStringList mission_titles, mission_versions, paths;
	QValueList<int> levels, lives, fuel_times_1000, scores_times_100;
	
	mission_titles = config->readListEntry ("mtitles");
	mission_versions = config->readListEntry ("mversions");
	paths = config->readPathListEntry ("mpaths");
	levels = config->readIntListEntry ("levels");
	lives = config->readIntListEntry ("lives");
	fuel_times_1000 = config->readIntListEntry ("fuel");
	scores_times_100 = config->readIntListEntry ("scores");
	
	num = mission_titles.count ();
	if (!((num == mission_versions.count ()) && (num == paths.count ()) && (num == levels.count ()) && (num == lives.count ()) && (num == fuel_times_1000.count ()) && (num == scores_times_100.count ()))) {
		cdp->debug_msg (i18n ("Count mismatch reading save game list"), Serious, 0);
		return;
	}
	
	GameDescription *descr;
	for (unsigned int i=0; i < num; ++i) {
		descr = new GameDescription;
		descr->mission_title = mission_titles[i];
		descr->mission_version = mission_versions[i];
		descr->mission_file = paths[i];
		descr->level = levels[i];
		descr->lives = lives[i];
		descr->fuel = fuel_times_1000[i] / 1000.0;
		descr->score = scores_times_100[i] / 100.0;

		saves_list.append (descr);
	}
}

void SaveGames::saveList () {
	KConfig *config = kapp->config();

	if (!config) {
		cdp->debug_msg (i18n ("Could not open config file while trying to save games"), Serious, 0);
		return;
	}
	
	config->setGroup ("SavedGames");

	QStringList mission_titles, mission_versions, paths;
	QValueList<int> levels, lives, fuel_times_1000, scores_times_100;
	
	for (GameDescription* descr = saves_list.first (); descr; descr = saves_list.next ()) {
		mission_titles.append (descr->mission_title);
		mission_versions.append (descr->mission_version);
		paths.append (descr->mission_file);
		levels.append (descr->level);
		lives.append (descr->lives);
		fuel_times_1000.append (static_cast<int> (descr->fuel * 1000));
		scores_times_100.append (static_cast<int> (descr->score * 100));
	}
	
	config->writeEntry ("mtitles", mission_titles);
	config->writeEntry ("mversions", mission_versions);
	config->writeEntry ("mpaths", paths);
	config->writeEntry ("levels", levels);
	config->writeEntry ("lives", lives);
	config->writeEntry ("fuel", fuel_times_1000);
	config->writeEntry ("scores", scores_times_100);
	
	config->sync();
}
