/***************************************************************************
                          settings  -  description
                             -------------------
    begin                : Tue Feb 3 2004
    copyright            : (C) 2004, 2005 by Thomas Friedrichsmeier
    email                : tfry@users.sourceforge.net
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/
#ifndef SETTINGS_H
#define SETTINGS_H

#include <qstring.h>
#include <qstringlist.h>

#include "tpbase.h"

/**
Manage (permanent) settings

@author Thomas Friedrichsmeier
*/
class Settings : public TPBase {
public:
    Settings();

    ~Settings();

public:
/** Returns true, if the commandline-option highlight was set, false otherwise */
	bool draw_highlight () { return draw_highlights; };
/** Returns true, if the commandline-option thorough_check was set, false otherwise */
	bool thorough_check() { return opt_thorough_check; };
/** Return the default mission path */
	QString defaultPath () { return prefix; };
/** Return the selected mission */
	QString getMission () { return selected_mission; };
/** Return calculations-per-second setting */
	int cps () { return pref_cps; };
/** Return frames-per-second setting */
	int fps () { return pref_fps; };
/** Return msecs-per-frame setting */
	int framePeriod () { return frame_period; };
/** Return animate_messages setting */
	bool animate_messages () { return pref_animate_messages; };
/** Return scrolling step setting */
	int scrollStep () { return pref_scroll_step; };
/** Signals, that the mission mission has just been opened (and therefore is the one
last chosen) */
	void setLastLoaded (const QString &mission);

/** Return list of known missions */
	QStringList getMissionList () { return known_missions; };
/** Returns true if this particular version of Taxipilot seems to be started for the fist time for this user */
	bool firstStartThisVersion () { return first_start_of_version; }
/** Returns true, if mission-selector should always be shown on startup */
	bool selectMissionOnStartup () { return pref_selectMissionOnStartup; }
/** is sound enabled? */
	bool sound_enabled () { return pref_sound; };
protected:
/** The debug_level */
	int debug_level;
/** prefix of the resource-dir, where the configuration-files are to be found */
	QString prefix;
/** Alternative prefix to the configuration-files */
	QString alt_prefix;
private:
/** Read commandline options */
	void readOptions ();
/** Read stored settings */
	void readSettings ();

friend class Preferences;
friend class MissionWidget;
friend class SavesWidget;

/** Saves the current config (and emits a config_changed ()) */
	void saveConfig ();

/** commandline options */
	bool draw_highlights;
/** commandline options */
	bool opt_thorough_check;

/** Preferences */
	int pref_fps;
/** 1000 / pref_fps. Used to avoid frequent divisions */
	int frame_period;
/** Preferences */
	int pref_cps;
/** Preferences */
	bool pref_animate_messages;
/** Preferences */
	int pref_scroll_step;
/** Preferences */
	bool pref_sound;

/** List of missions known to Taxipilot */
	QStringList known_missions;
/** Selected mission */
	QString selected_mission;
/** Preferences */
	bool pref_selectMissionOnStartup;
/** This Version of Taxipilot appears to be started for the first time */
	bool first_start_of_version;

friend class Taxipilot;
/** data of saved game just loaded (if any). These do NOT get saved */
	int saved_lives;
	int saved_level;
	float saved_fuel;
};

#endif
