/***************************************************************************
                          soundwrapper.h  -  description
                             -------------------
    begin                : Sam Mai 25 2002
    copyright            : (C) 2002 by Thomas Friedrichsmeier
    email                : tfry@users.sourceforge.net
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef SOUNDWRAPPER_H
#define SOUNDWRAPPER_H

#include <arts/soundserver.h>
#include <arts/kartsdispatcher.h>
#include <qmap.h>
#include <qvaluelist.h>

#include <qstring.h>

#include "EXT_wavpo/ext_wav_play_object.h"
#include "tpbase.h"

/**
  *@author Thomas Friedrichsmeier
  */

using namespace Arts;

class SoundWrapper : public TPBase {
public: 
	SoundWrapper();
	~SoundWrapper();
/** Start a sound by it's id (sound has to be registered first!). Pitch is given in relation regular pitch */
	void start_sound (int id, float pitch=1);
/** Stop a sound by it's id. Leaves the sound registered */
	void stop_sound (int id);
private:
	friend class SoundHandle;
/** Register a sound with given filename. If continuous, will loop until stopped. If immediate_interrupt, stopping the sound will stop it immediately (instead of simply not playing
a looping sound again). Returns an id to adress this sound. Use SoundHandle instead of trying to call this function directly. */
	int register_sound (const QString &name, bool continuous, bool immediate_interrupt);
/** Unregisters a sound, i.e. frees the sounds resources. Should be done whenever a sound will not be needed again.
Use SoundHandle instead of trying to call this function directly. */
	void unregister_sound (int id);
	struct Sound {
	/** We need to store the filename in order to be able to delete and recreate the sound when sound is turned on/off */
		QString filename;
		EXT_WavPlayObject po;
		Arts::Synth_AMAN_PLAY player;
		bool continuous;
		bool immediate_interrupt;
	/** whether the sound was (successfully) created */
		bool created;
	};

	typedef QMap<int, Sound> SoundMap;
	SoundMap sound_map;

/** Creates server, factory and all registered sounds */
	void start_server ();
/** Discards server, factory and all created registered sounds */
	void stop_server ();
/** Creates a single sound */
	void create_sound (SoundMap::Iterator it);
/** Discards a single (created, registered) sound */
	void discard_sound (SoundMap::Iterator it);

	bool sound_enabled;
	bool sound;
	bool sound_error;
	SoundServerV2 server;

	QValueList<int> freed_ids;

	friend class Taxipilot;
/** Inits (or reconfigures) the soundwrapper. To be called after construction and whenever relevant parameters change. */
	void init (bool enable_sound);
};

#endif
